package org.amse.shElena.toyRec.sampleBase;

import java.io.File;

import org.amse.shElena.toyRec.samples.ISample;

/**
 * Provides methods to deal with samples in their repository.
 */
public interface ISampleBase extends Iterable<ISample> {
	/**
	 * Adds the sample to the base.
	 * 
	 * @param sample
	 *            sample to add.
	 */
	void addSample(ISample sample);

	/**
	 * Removes the first occurrence in this base of the specified element. If
	 * this base does not contain the element, it is unchanged.
	 * 
	 * @param sample
	 *            sample to remove.
	 */
	void removeSample(ISample sample);

	/**
	 * Adds all samples of given base.
	 * 
	 * @param base
	 *            base to add.
	 */
	void addSampleBase(ISampleBase base);

	/**
	 * Method to check if the base is empty.
	 * 
	 * @return true if there are no samples in the base, false otherwise.
	 */
	boolean isEmpty();

	/**
	 * Size of the base.
	 * 
	 * @return number of samples in the base.
	 */
	int size();

	/**
	 * Loads base from the given file.
	 * 
	 * @param file
	 *            file to load from.
	 */
	void loadSampleBase(File file);

	/**
	 * Saves base to the file.
	 * 
	 * @param file
	 *            file to save the base.
	 */
	void saveSampleBase(File file);

	/**
	 * 
	 * @return array of all samples in the base.
	 */
	ISample[] getSamples();

	/**
	 * Removes all samples.
	 */
	void clear();
}
